#!/bin/bash
server_url=$1
deploy_key=$2

if [ $# -lt 2 ]
    then
        echo "Wrong number of arguments supplied."
        echo "Usage: $0 <server_url> <deploy_key>."
        exit 1
fi

register(){
server_url=$1
deploy_key=$2
honeypot=$3
hostname=$(hostname)

curl -s -X POST -H "Content-Type: application/json" -d "{
        \"name\": \"${hostname}-${honeypot}\", 
        \"hostname\": \"$hostname\", 
        \"deploy_key\": \"$deploy_key\",
        \"honeypot\": \"$honeypot\"
}" $server_url/api/sensor/ > /tmp/deploy.json

uuid=$(python -c 'import json;obj=json.load(file("/tmp/deploy.json"));print obj["uuid"]')

if [ -z "$uuid" ]
    then
        echo "Could not create sensor using name \"$hostname\"."
        exit 1
fi

set -e

echo "Created sensor: " $uuid

######################################################
# hpfeeds info
export HPF_HOST=$(echo $server_url | sed 's#^http://##; s#^https://##; s#/.*$##; s/:.*$//')
export HPF_PORT="10000"
export HPF_IDENT=$(python -c 'import json;obj=json.load(file("/tmp/deploy.json"));print obj["identifier"]')
export HPF_SECRET=$(python -c 'import json;obj=json.load(file("/tmp/deploy.json"));print obj["secret"]')
######################################################
}

register $server_url $deploy_key "amun"


# Setup HPFeeds
cat > /usr/share/amun/conf/log-hpfeeds.conf <<EOF

[database_hpfeeds]
server = $HPF_HOST
port = $HPF_PORT
identifier = $HPF_IDENT
secret = $HPF_SECRET
debug = 0

EOF

systemctl restart amun

